function [V,N_Na,Istim,t] = stochasticHH(modelname,I0)

% stochasticHH runs a simulation of a stochastic Hodgkin-Huxley model of:
%
%    Bruce, I. C. (2007). "Implementation issues in approximate methods for
%    stochastic HodgkinHuxley models," Annals of Biomedical Engineering
%    35(2):315318.
%
% [V,N_Na,Istim,t] = stochasticHH(modelname,I0)
%
% V is the relative transmembrane potential in units of millivolt.
% N_Na is the number of open sodium channels.
% Istim is the stimulus waveform in units of microampere.
% t is the time in units of millisecond.
%
% modelname is the model type, which may be one of the following:-
%    - 'hhfloor': Deterministic Hodgkin-Huxley model with N_Na rounded down.
%    - 'hhnint':  Deterministic Hodgkin-Huxley model with N_Na rounded to
%                 nearest integer.
%    - 'fxfloor': Fox algorithm with N_Na rounded down.
%    - 'fxnint':  Fox algorithm with N_Na rounded to nearest integer.
%    - 'cw':      Chow & White algorithm.
% I0 is the current amplitude in units of picoampere.  A single value
%    (e.g., I0 = 22) produces a single pulse of duration 100 microsecond
%    and amplitude I0 pA.
%    A pair of values (e.g., I0 = [22 35]) produces a conditioning prepulse
%    of duration 500 microsecond & amplitude I0(1) pA and a pulse of
%    duration 100 microsecond & amplitude I0(2) pA.
%
% Ian C. Bruce (ibruce@ieee.org), Faheem Dinath & Melissa T. Perri  2007

% Set up the appropriate functions for the chosen model
if (strcmp(modelname,'hhfloor'))
    calcN_Na = @hhfloor;
    initN_Na = @inithhfloor;
    global m h;
elseif (strcmp(modelname,'hhnint'))
    calcN_Na = @hhnint;
    initN_Na = @inithhnint;
    global m h;
elseif (strcmp(modelname,'fxfloor'))
    calcN_Na = @fxfloor;
    initN_Na = @initfxfloor;
    global m h;
elseif (strcmp(modelname,'fxnint'))
    calcN_Na = @fxnint;
    initN_Na = @initfxnint;
    global m h;
elseif (strcmp(modelname,'cw'))
    calcN_Na = @cw;
    initN_Na = @initcw;
    global Nmh;
else
    error(['Model type "' modelname '" unknown'])
end

% Constant variables used in simulations.  Corrected values are given in
% Table 1 of Bruce (2007)
dt = 1e-3; % Euler integration step in ms [dt = 1us]
sim_time = 2; % simulation time in ms
gNa = 2.569e-8; % maximum sodium conductance in mS [gamma_Na = 25.69 pS/channel]
Na_max = 1000; % total number of sodium channels
Cm = 0.0714e-6; % membrane capacitance in uF [Cm = 0.0714nF]
Rm = 1953.49e3; % membrane resistance in kOhms [Rm = 1.95349e3 MOhms]
V_rest = -78; % membrane resting potential in mV
E_Na = 66 - V_rest; % relative sodium Nernst potential in mV

t = 0:dt:sim_time; % time vector in ms
max_k = length(t); % number of Euler integration steps

% Create output variables
V = zeros(1,max_k); % Relative transmembrane potential vector
N_Na = zeros(1,max_k); % Open sodium channel count vector
Istim = zeros(1,max_k); % Stimulus vector

% Create stimulus vector for stimuli of Mino et al. (2002) and Bruce (2007)
switch length(I0)
    case 1
        Istim(1:round(0.1/dt-1)) = I0*1e-6;       % microampere     monophasic pulse
    case 2
        Istim(1:round(0.5/dt-1)) = I0(1)*1e-6;    % microampere     preconditioning pulse
        Istim(round(0.5/dt):round(0.6/dt-1)) = I0(2)*1e-6;  % uA    monophasic pulse
    otherwise
        error('Current pulse configuration not accepted')
end

% Calculate sodium gating particle transition rates for initial (resting)
% membrane potential
markovrates = calcrates(V(1));

% Calculate initial number of open sodium channels
N_Na(1) = initN_Na(markovrates,Na_max,dt);

for k = 1:(max_k-1)

    % Calculate sodium gating particle transition rates for present
    % membrane potential
    markovrates = calcrates(V(k));
    
    % Calculate number of open sodium channels for next Euler time step
    N_Na(k+1) = calcN_Na(markovrates,Na_max,dt);
    
    % Calculate membrane potential for next Euler time step
    V(k+1) = V(k)+(Istim(k) - V(k)/Rm - gNa*N_Na(k)*(V(k)-E_Na))*dt/Cm;
    
end

% end of main function stochasticHH

function markovrates=calcrates(V)
% Calculate sodium gating particle transition rates according to Eqns.
% (2)-(5) of Mino et al. (2002)

    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));
    
% end of subfunction calcrates

function N_Na=inithhfloor(markovrates,Na_max,dt)
% Calculate initial number of open sodium channels according to Eqn. (7)
% of Mino et al. (2002) with floor(N_Na)

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);

    N_Na = floor(Na_max*m^3*h);

% end of subfunction inithhfloor

function N_Na=hhfloor(markovrates,Na_max,dt)
% Calculate number of open sodium channels according to Eqns. (7) & (8)
% of Mino et al. (2002) with floor(N_Na)

    global m h;

    m = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt;
    h = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt;

    N_Na = floor(Na_max*m^3*h);

% end of subfunction hhfloor

function N_Na=inithhnint(markovrates,Na_max,dt)
% Calculate initial number of open sodium channels according to Eqn. (7)
% of Mino et al. (2002) with nint(N_Na)

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);

    N_Na = round(Na_max*m^3*h);

% end of subfunction inithhnint

function N_Na=hhnint(markovrates,Na_max,dt)
% Calculate number of open sodium channels according to Eqns. (7) & (8)
% of Mino et al. (2002) with nint(N_Na)

    global m h;

    m = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt;
    h = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt;

    N_Na = round(Na_max*m^3*h);

% end of subfunction hhnint

function N_Na=initfxfloor(markovrates,Na_max,dt)
% Calculate initial number of open sodium channels according to Eqns.
% (9)-(13) of Mino et al. (2002) with floor(N_Na)

    global m h;

    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);

    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        mnew = -1;
        while (mnew<0)||(mnew>1)
            mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt);
        end
        m = mnew;
        
        hnew = -1;
        while (hnew<0)||(hnew>1)
            hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt);
        end
        h = hnew;
        
    end

    N_Na = floor(Na_max*m^3*h);
    
% end of subfunction initfxfloor

function N_Na=fxfloor(markovrates,Na_max,dt)
% Calculate number of open sodium channels according to Eqns. (9)-(13)
% of Mino et al. (2002) with floor(N_Na)

    global m h;

    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt);
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt);
    end
    h = hnew;
    
    N_Na = floor(Na_max*m^3*h);

% end of subfunction fxfloor

function N_Na=initfxnint(markovrates,Na_max,dt)
% Calculate initial number of open sodium channels according to Eqns.
% (9)-(13) of Mino et al. (2002) with nint(N_Na)

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);

    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        mnew = -1;
        while (mnew<0)||(mnew>1)
            mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt);
        end
        m = mnew;
        
        hnew = -1;
        while (hnew<0)||(hnew>1)
            hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt);
        end
        h = hnew;
        
    end

    N_Na = round(Na_max*m^3*h);
    
% end of subfunction initfxnint

function N_Na=fxnint(markovrates,Na_max,dt)
% Calculate number of open sodium channels according to Eqns. (9)-(13)
% of Mino et al. (2002) with nint(N_Na)

    global m h;

    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt);
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt);
    end
    h = hnew;
    
    N_Na = round(Na_max*m^3*h);

% end of subfunction fxnint

function N_Na=initcw(markovrates,Na_max,dt)
% Calculate initial number of open sodium channels according to Eqn. (17)
% and Eqns. (A1)-(A6) of Mino et al. (2002)

    global Nmh;  
   
    % Set initial number of channels in each state
    if Na_max == 1e3 % Mean steady-state values known for Na_max = 1000
        Nmh = [246     6     0     0   731    17     0     0];
        settletime = 100; % Number of Euluer time steps over which to randomize initial conditions
    else % otherwise determine approximately appropriate initial values
        % Calculate initial steady-state fraction of open h particles
        hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
        Nmh = zeros(1,8);
        Nmh(5) = round(Na_max*hinf);
        Nmh(1) = Na_max - Nmh(5);
        settletime = 200; % Number of Euluer time steps over which to randomize initial conditions
    end

    % Calculate transition rates of "escapes"
    zeta = [3*markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah + 2*markovrates.am;...
            2*markovrates.bm + markovrates.ah + markovrates.am;...
            3*markovrates.bm + markovrates.ah;...
            3*markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh + 2*markovrates.am;...
            2*markovrates.bm + markovrates.bh + markovrates.am;...
            3*markovrates.bm + markovrates.bh];

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,2,3,3,4,1,5,2,6,3,7,4,8,5,6,6,7,7,8];
    p_trans_next = [0,2,1,3,2,4,3,5,1,6,2,7,3,8,4,6,5,7,6,8,7];
        
    t_l_total = 0; % cumulative transition lifetime

    Nmh_ensemble = Nmh;
    
    % Randomize initial conditions
    while (t_l_total<=settletime*dt)
        
        lambda = Nmh*zeta;
    
        t_l = - log(rand(1)) / lambda; % Determine transition lifetime
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l; 
        
        % If cumulative transition lifetime is long enough, return initial
        % number of channels in each state
         if (t_l_total>settletime*dt)
            
            break;
            
        end

        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            3*markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            2*markovrates.am*Nmh(2),...
            2*markovrates.bm*Nmh(3),...
            markovrates.am*Nmh(3),...
            3*markovrates.bm*Nmh(4),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(5),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(6),...
            markovrates.ah*Nmh(3),...
            markovrates.bh*Nmh(7),...
            markovrates.ah*Nmh(4),...
            markovrates.bh*Nmh(8),...
            3*markovrates.am*Nmh(5),...
            markovrates.bm*Nmh(6),...
            2*markovrates.am*Nmh(6),...
            2*markovrates.bm*Nmh(7),...
            markovrates.am*Nmh(7),...
            3*markovrates.bm*Nmh(8)]/lambda);
        
        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state                     

        Nmh_ensemble = [Nmh_ensemble; Nmh];
        
    end

    N_Na = Nmh(end);

% end of subfunction initcw

function N_Na=cw(markovrates,Na_max,dt)

    global Nmh;
    
    % Calculate transition rates of "escapes"
    zeta = [3*markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah + 2*markovrates.am;...
            2*markovrates.bm + markovrates.ah + markovrates.am;...
            3*markovrates.bm + markovrates.ah;...
            3*markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh + 2*markovrates.am;...
            2*markovrates.bm + markovrates.bh + markovrates.am;...
            3*markovrates.bm + markovrates.bh];

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,2,3,3,4,1,5,2,6,3,7,4,8,5,6,6,7,7,8];
    p_trans_next = [0,2,1,3,2,4,3,5,1,6,2,7,3,8,4,6,5,7,6,8,7];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = Nmh*zeta;
    
        t_l = - log(rand(1)) / lambda; % Determine transition lifetime
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            3*markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            2*markovrates.am*Nmh(2),...
            2*markovrates.bm*Nmh(3),...
            markovrates.am*Nmh(3),...
            3*markovrates.bm*Nmh(4),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(5),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(6),...
            markovrates.ah*Nmh(3),...
            markovrates.bh*Nmh(7),...
            markovrates.ah*Nmh(4),...
            markovrates.bh*Nmh(8),...
            3*markovrates.am*Nmh(5),...
            markovrates.bm*Nmh(6),...
            2*markovrates.am*Nmh(6),...
            2*markovrates.bm*Nmh(7),...
            markovrates.am*Nmh(7),...
            3*markovrates.bm*Nmh(8)]/lambda);

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        
    end
        
    N_Na = Nmh(end);

% end of subfunction cw
